package com.sun.org.apache.xml.internal.security.c14n.implementations;

import com.sun.org.slf4j.internal.LoggerFactory;
import java.net.URI;
import java.util.Iterator;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Collection;
import org.w3c.dom.Attr;
import java.util.ArrayList;
import java.util.List;
import com.sun.org.slf4j.internal.Logger;

class XmlAttrStack
{
    private static final Logger LOG;
    private int currentLevel;
    private int lastlevel;
    private XmlAttrStack.XmlsStackElement cur;
    private List<XmlAttrStack.XmlsStackElement> levels;
    private boolean c14n11;
    
    public XmlAttrStack(boolean c14n11) {
        this.currentLevel = 0;
        this.lastlevel = 0;
        this.levels = new ArrayList();
        this.c14n11 = c14n11;
    }
    
    void push(int currentLevel) {
        this.currentLevel = currentLevel;
        if (this.currentLevel == -1) {
            return;
        }
        this.cur = null;
        while (this.lastlevel >= this.currentLevel) {
            this.levels.remove(this.levels.size() - 1);
            int size = this.levels.size();
            if (size == 0) {
                this.lastlevel = 0;
                return;
            }
            this.lastlevel = ((XmlAttrStack.XmlsStackElement)this.levels.get(size - 1)).level;
        }
        return;
    }
    
    void addXmlnsAttr(Attr attr) {
        if (this.cur == null) {
            this.cur = new XmlAttrStack.XmlsStackElement();
            this.cur.level = this.currentLevel;
            this.levels.add(this.cur);
            this.lastlevel = this.currentLevel;
        }
        this.cur.nodes.add(attr);
    }
    
    void getXmlnsAttr(Collection<Attr> collection) {
        int i = this.levels.size() - 1;
        if (this.cur == null) {
            this.cur = new XmlAttrStack.XmlsStackElement();
            this.cur.level = this.currentLevel;
            this.lastlevel = this.currentLevel;
            this.levels.add(this.cur);
        }
        boolean n = false;
        if (i == -1) {
            n = true;
        }
        else {
            XmlAttrStack.XmlsStackElement xmlsStackElement = (XmlAttrStack.XmlsStackElement)this.levels.get(i);
            if (xmlsStackElement.rendered && xmlsStackElement.level + 1 == this.currentLevel) {
                n = true;
            }
        }
        if (!(n == 0)) {
            collection.addAll(this.cur.nodes);
            this.cur.rendered = true;
            return;
        }
        HashMap o = new HashMap();
        if (this.c14n11) {
            ArrayList o2 = new ArrayList();
            boolean n2 = true;
            while (i >= 0) {
                XmlAttrStack.XmlsStackElement xmlsStackElement2 = (XmlAttrStack.XmlsStackElement)this.levels.get(i);
                if (xmlsStackElement2.rendered) {
                    n2 = false;
                }
                Iterator<Attr> iterator2 = xmlsStackElement2.nodes.iterator();
                while (iterator2.hasNext() && n2) {
                    Attr attr2 = (Attr)iterator2.next();
                    if (attr2.getLocalName().equals("base") && !xmlsStackElement2.rendered) {
                        o2.add(attr2);
                    }
                    else if (!o.containsKey(attr2.getName())) {
                        o.put(attr2.getName(), attr2);
                    }
                    else {
                        continue;
                    }
                }
                --i;
            }
            if (!o2.isEmpty()) {
                Iterator<Object> iterator3 = collection.iterator();
                String value = null;
                Attr attr3 = null;
                while (iterator3.hasNext()) {
                    Attr attr4 = (Attr)iterator3.next();
                    if (attr4.getLocalName().equals("base")) {
                        value = attr4.getValue();
                        attr3 = attr4;
                        break;
                    }
                    continue;
                }
                for (Attr attr5 : o2) {
                    if (value == null) {
                        value = attr5.getValue();
                        attr3 = attr5;
                    }
                    else {
                        try {
                            value = joinURI(attr5.getValue(), value);
                        }
                        catch (URISyntaxException e) {
                            XmlAttrStack.LOG.debug(e.getMessage(), e);
                        }
                    }
                }
                if (value != null && value.length() != 0) {
                    attr3.setValue(value);
                    collection.add(attr3);
                }
            }
        }
        else {
            while (i >= 0) {
                for (Attr attr : ((XmlAttrStack.XmlsStackElement)this.levels.get(i)).nodes) {
                    if (!o.containsKey(attr.getName())) {
                        o.put(attr.getName(), attr);
                    }
                    else {
                        continue;
                    }
                }
                --i;
            }
        }
        this.cur.rendered = true;
        collection.addAll(o.values());
        return;
    }
    
    private static String joinURI(String string, String str) throws URISyntaxException {
        String scheme = null;
        String authority = null;
        String path = "";
        String query = null;
        if (string != null) {
            if (string.endsWith("..")) {
                string += "/";
            }
            URI o = new URI(string);
            scheme = o.getScheme();
            authority = o.getAuthority();
            path = o.getPath();
            query = o.getQuery();
        }
        URI o2 = new URI(str);
        String scheme2 = o2.getScheme();
        String authority2 = o2.getAuthority();
        String path2 = o2.getPath();
        String query2 = o2.getQuery();
        if (scheme2 != null && scheme2.equals(scheme)) {
            scheme2 = null;
        }
        String path3;
        if (scheme2 != null) {
            path3 = removeDotSegments(path2);
        }
        else {
            if (authority2 != null) {
                path3 = removeDotSegments(path2);
            }
            else {
                if (path2.length() == 0) {
                    path3 = path;
                    if (query2 != null) {
                    }
                    else {
                        query2 = query;
                    }
                }
                else {
                    if (path2.startsWith("/")) {
                        path3 = removeDotSegments(path2);
                    }
                    else {
                        String s;
                        if (authority != null && path.length() == 0) {
                            s = "/" + path2;
                        }
                        else {
                            int lastIndex = path.lastIndexOf(47);
                            if (lastIndex == -1) {
                                s = path2;
                            }
                            else {
                                s = path.substring(0, lastIndex + 1) + path2;
                            }
                        }
                        path3 = removeDotSegments(s);
                    }
                }
                authority2 = authority;
            }
            scheme2 = scheme;
        }
        return new URI(scheme2, authority2, path3, query2, null).toString();
    }
    
    private static String removeDotSegments(String s) {
        XmlAttrStack.LOG.debug("STEP OUTPUT BUFFER\t\tINPUT BUFFER");
        String s2;
        for (s2 = s; s2.indexOf("//") > -1; s2 = s2.replaceAll("//", "/")) {}
        StringBuilder o = new StringBuilder();
        if (s2.charAt(0) == '/') {
            o.append("/");
            s2 = s2.substring(1);
        }
        printStep("1 ", o.toString(), s2);
        while (s2.length() != 0) {
            if (s2.startsWith("./")) {
                s2 = s2.substring(2);
                printStep("2A", o.toString(), s2);
            }
            else if (s2.startsWith("../")) {
                s2 = s2.substring(3);
                if (!o.toString().equals("/")) {
                    o.append("../");
                }
                printStep("2A", o.toString(), s2);
            }
            else if (s2.startsWith("/./")) {
                s2 = s2.substring(2);
                printStep("2B", o.toString(), s2);
            }
            else if (s2.equals("/.")) {
                s2 = s2.replaceFirst("/.", "/");
                printStep("2B", o.toString(), s2);
            }
            else if (s2.startsWith("/../")) {
                s2 = s2.substring(3);
                if (o.length() == 0) {
                    o.append("/");
                }
                else if (o.toString().endsWith("../")) {
                    o.append("..");
                }
                else if (o.toString().endsWith("..")) {
                    o.append("/..");
                }
                else {
                    int lastIndex2 = o.lastIndexOf("/");
                    if (lastIndex2 == -1) {
                        o = new StringBuilder();
                        if (s2.charAt(0) == '/') {
                            s2 = s2.substring(1);
                        }
                    }
                    else {
                        o = o.delete(lastIndex2, o.length());
                    }
                }
                printStep("2C", o.toString(), s2);
            }
            else if (s2.equals("/..")) {
                s2 = s2.replaceFirst("/..", "/");
                if (o.length() == 0) {
                    o.append("/");
                }
                else if (o.toString().endsWith("../")) {
                    o.append("..");
                }
                else if (o.toString().endsWith("..")) {
                    o.append("/..");
                }
                else {
                    int lastIndex = o.lastIndexOf("/");
                    if (lastIndex == -1) {
                        o = new StringBuilder();
                        if (s2.charAt(0) == '/') {
                            s2 = s2.substring(1);
                        }
                    }
                    else {
                        o = o.delete(lastIndex, o.length());
                    }
                }
                printStep("2C", o.toString(), s2);
            }
            else if (s2.equals(".")) {
                s2 = "";
                printStep("2D", o.toString(), s2);
            }
            else if (s2.equals("..")) {
                if (!o.toString().equals("/")) {
                    o.append("..");
                }
                s2 = "";
                printStep("2D", o.toString(), s2);
            }
            else {
                int index = s2.indexOf(47);
                int index2;
                if (index == 0) {
                    index2 = s2.indexOf(47, 1);
                }
                else {
                    index2 = index;
                    index = 0;
                }
                String str;
                if (index2 == -1) {
                    str = s2.substring(index);
                    s2 = "";
                }
                else {
                    str = s2.substring(index, index2);
                    s2 = s2.substring(index2);
                }
                o.append(str);
                printStep("2E", o.toString(), s2);
            }
        }
        if (o.toString().endsWith("..")) {
            o.append("/");
            printStep("3 ", o.toString(), s2);
        }
        return o.toString();
    }
    
    private static void printStep(String str, String str2, String s) {
        if (XmlAttrStack.LOG.isDebugEnabled()) {
            XmlAttrStack.LOG.debug(" " + str + ":   " + str2);
            if (str2.length() == 0) {
                XmlAttrStack.LOG.debug("\t\t\t\t" + s);
            }
            else {
                XmlAttrStack.LOG.debug("\t\t\t" + s);
            }
        }
    }
    
    static {
        LOG = LoggerFactory.getLogger(XmlAttrStack.class);
    }
    
    static class XmlsStackElement
    {
        int level;
        boolean rendered;
        List<Attr> nodes;
        
        XmlsStackElement() {
            this.rendered = false;
            this.nodes = new ArrayList();
        }
    }
}
